"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PropertyClient = void 0;
const anchor_1 = require("@project-serum/anchor");
const common_sdk_1 = require("@orca-so/common-sdk");
const __1 = require("..");
const spl_token_1 = require("@solana/spl-token");
const bignumber_js_1 = __importDefault(require("bignumber.js"));
class PropertyClient {
    constructor(ctx, propertyMintAccount, propertyData, uiPropertyData, pda) {
        this.ctx = ctx;
        this.propertyMintAccount = propertyMintAccount;
        this.propertyData = propertyData;
        this.uiPropertyData = uiPropertyData;
        this.pda = pda;
    }
    static getProperty(ctx, propertyMintAccount) {
        return __awaiter(this, void 0, void 0, function* () {
            const pda = new __1.PDA(ctx.program.programId);
            const property = pda.property(propertyMintAccount);
            const propertyData = yield ctx.fetcher.getProperty(property.key, true);
            if (!propertyData) {
                throw new Error(`1, Property of ${propertyMintAccount} not found`);
            }
            const uiPropertyData = yield convertPropertyDataToUiDataAndExtendData(ctx, pda, propertyData);
            return new PropertyClient(ctx, propertyMintAccount, propertyData, uiPropertyData, pda);
        });
    }
    // Only admin authority
    static new(ctx, admin, propertyMintAccount, privateSaleStartTime, privateSaleEndTime, privateSaleSupplyAmount, privateSaleTokenPrice, privateSaleMinimumPurchaseAmount, privateSaleCommissionAmount, privateSaleReferralCommissionAmount, privateSaleDiscountPercentage, privateSaleTgePercentage, privateSaleUnlockPeriod, privateSaleUnlockDuration, privateSaleMinimumPlatformAmount, publicSaleStartTime, publicSaleEndTime, publicSaleSupplyAmount, publicSaleTokenPrice, publicSaleMinimumPurchaseAmount, publicSaleCommissionAmount, publicSaleCommissionBoostAmount, publicSaleReferralCommissionAmount, publicSaleReferralCommissionBoostAmount, publicSaleReferralCommissionBoostThreshold) {
        return __awaiter(this, void 0, void 0, function* () {
            const pda = new __1.PDA(ctx.program.programId);
            const platform = pda.platform();
            const property = pda.property(propertyMintAccount);
            const platformData = yield ctx.fetcher.getPlatform(platform.key, true);
            if (!platformData) {
                throw new Error(`2, Platform not found`);
            }
            const tx = new common_sdk_1.TransactionBuilder(ctx.provider.connection, ctx.provider.wallet);
            const bump = property.bump;
            const propertyVaultTokenAccount = yield spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, propertyMintAccount, property.key, true);
            const platformMintAccount = platformData.platformMintAccount;
            const platformVaultTokenAccount = yield spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, platformMintAccount, property.key, true);
            const createPropertyIx = yield ctx.methods.createProperty({
                accounts: {
                    admin,
                    platformState: platform.key,
                    propertyState: property.key,
                    propertyMintAccount,
                    propertyVaultTokenAccount,
                    platformMintAccount,
                    platformVaultTokenAccount,
                },
                inputs: {
                    privateSaleStartTime,
                    privateSaleEndTime,
                    privateSaleSupplyAmount,
                    privateSaleTokenPrice,
                    privateSaleMinimumPurchaseAmount,
                    privateSaleCommissionAmount,
                    privateSaleReferralCommissionAmount,
                    privateSaleDiscountPercentage,
                    privateSaleTgePercentage,
                    privateSaleUnlockPeriod,
                    privateSaleUnlockDuration,
                    privateSaleMinimumPlatformAmount,
                    publicSaleStartTime,
                    publicSaleEndTime,
                    publicSaleSupplyAmount,
                    publicSaleTokenPrice,
                    publicSaleMinimumPurchaseAmount,
                    publicSaleCommissionAmount,
                    publicSaleCommissionBoostAmount,
                    publicSaleReferralCommissionAmount,
                    publicSaleReferralCommissionBoostAmount,
                    publicSaleReferralCommissionBoostThreshold,
                    bump,
                },
            });
            if (createPropertyIx.ix) {
                tx.addInstruction(createPropertyIx.ix);
            }
            console.log("New property token: ", propertyMintAccount.toBase58());
            console.log("New property state: ", property.key.toBase58());
            console.log("Vault of property token of property: ", propertyVaultTokenAccount.toBase58());
            console.log("Vault of PROP token of property: ", platformVaultTokenAccount.toBase58());
            return tx;
        });
    }
    // Only admin authority
    update(ctx, admin, propertyMintAccount, privateSaleStartTime, privateSaleEndTime, privateSaleSupplyAmount, privateSaleTokenPrice, privateSaleMinimumPurchaseAmount, privateSaleCommissionAmount, privateSaleReferralCommissionAmount, privateSaleDiscountPercentage, privateSaleTgePercentage, privateSaleUnlockPeriod, privateSaleUnlockDuration, privateSaleMinimumPlatformAmount, publicSaleStartTime, publicSaleEndTime, publicSaleSupplyAmount, publicSaleTokenPrice, publicSaleMinimumPurchaseAmount, publicSaleCommissionAmount, publicSaleCommissionBoostAmount, publicSaleReferralCommissionAmount, publicSaleReferralCommissionBoostAmount, publicSaleReferralCommissionBoostThreshold) {
        return __awaiter(this, void 0, void 0, function* () {
            const pda = this.pda;
            const platform = pda.platform();
            const property = pda.property(propertyMintAccount);
            const platformData = yield ctx.fetcher.getPlatform(platform.key, true);
            if (!platformData) {
                throw new Error(`3, Platform not found`);
            }
            const tx = new common_sdk_1.TransactionBuilder(ctx.provider.connection, ctx.provider.wallet);
            const updatePropertyIx = yield ctx.methods.updateProperty({
                accounts: {
                    admin,
                    platformState: platform.key,
                    propertyState: property.key,
                    propertyMintAccount,
                },
                inputs: {
                    privateSaleStartTime,
                    privateSaleEndTime,
                    privateSaleSupplyAmount,
                    privateSaleTokenPrice,
                    privateSaleMinimumPurchaseAmount,
                    privateSaleCommissionAmount,
                    privateSaleReferralCommissionAmount,
                    privateSaleDiscountPercentage,
                    privateSaleTgePercentage,
                    privateSaleUnlockPeriod,
                    privateSaleUnlockDuration,
                    privateSaleMinimumPlatformAmount,
                    publicSaleStartTime,
                    publicSaleEndTime,
                    publicSaleSupplyAmount,
                    publicSaleTokenPrice,
                    publicSaleMinimumPurchaseAmount,
                    publicSaleCommissionAmount,
                    publicSaleCommissionBoostAmount,
                    publicSaleReferralCommissionAmount,
                    publicSaleReferralCommissionBoostAmount,
                    publicSaleReferralCommissionBoostThreshold,
                },
            });
            if (updatePropertyIx.ix) {
                tx.addInstruction(updatePropertyIx.ix);
            }
            console.log("Update property token: ", propertyMintAccount.toBase58());
            console.log("Update property state: ", property.key.toBase58());
            return tx;
        });
    }
    // Only admin authority
    withdrawToken(ctx, admin, propertyMintAccount, withdrawMintAccount, amount) {
        return __awaiter(this, void 0, void 0, function* () {
            const pda = this.pda;
            const platform = pda.platform();
            const property = pda.property(propertyMintAccount);
            const withdrawVaultTokenAccount = yield spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, withdrawMintAccount, property.key, true);
            const adminWithdrawTokenAccount = yield spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, withdrawMintAccount, admin, true);
            const tx = new common_sdk_1.TransactionBuilder(ctx.provider.connection, ctx.provider.wallet);
            const adminWithdrawTokenAccountInfo = yield ctx.connection.getAccountInfo(adminWithdrawTokenAccount);
            if (adminWithdrawTokenAccountInfo == null) {
                tx.addInstruction({
                    instructions: [
                        spl_token_1.Token.createAssociatedTokenAccountInstruction(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, withdrawMintAccount, adminWithdrawTokenAccount, admin, ctx.wallet.publicKey),
                    ],
                    cleanupInstructions: [],
                    signers: [],
                });
            }
            const withdrawTokenIx = yield ctx.methods.withdrawToken({
                accounts: {
                    admin,
                    platformState: platform.key,
                    propertyState: property.key,
                    propertyMintAccount,
                    withdrawMintAccount,
                    withdrawVaultTokenAccount,
                    adminWithdrawTokenAccount,
                },
                inputs: {
                    amount,
                },
            });
            if (withdrawTokenIx.ix) {
                tx.addInstruction(withdrawTokenIx.ix);
            }
            console.log("Withdraw token: ", withdrawMintAccount.toBase58());
            console.log("Withdraw to: ", admin.toBase58());
            return tx;
        });
    }
    purchasePropertyToken(purchaseAmount, referrer) {
        return __awaiter(this, void 0, void 0, function* () {
            const tx = new common_sdk_1.TransactionBuilder(this.ctx.provider.connection, this.ctx.provider.wallet);
            const user = this.ctx.provider.publicKey;
            const pda = this.pda;
            const platformData = yield this.ctx.fetcher.getPlatform(pda.platform().key, true);
            if (!platformData) {
                throw new Error(`4, Platform state not found`);
            }
            const admin = platformData.admin;
            const purchaseMintAccount = platformData.purchaseMintAccount;
            const platformMintAccount = platformData.platformMintAccount;
            const purchaseMintAccountInfo = yield this.ctx.connection.getParsedAccountInfo(purchaseMintAccount);
            if (purchaseMintAccountInfo.value == null) {
                throw new Error(`5, Info of ${purchaseMintAccount} not found`);
            }
            const purchaseDecimals = purchaseMintAccountInfo.value.data.parsed.info
                .decimals;
            const adminPurchaseTokenAccount = yield spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, purchaseMintAccount, admin, true);
            const adminPurchaseTokenAccountInfo = yield this.ctx.connection.getAccountInfo(adminPurchaseTokenAccount, "confirmed");
            if (adminPurchaseTokenAccountInfo == null) {
                tx.addInstruction({
                    instructions: [
                        spl_token_1.Token.createAssociatedTokenAccountInstruction(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, purchaseMintAccount, adminPurchaseTokenAccount, admin, user),
                    ],
                    cleanupInstructions: [],
                    signers: [],
                });
            }
            const userPurchaseTokenAccount = yield spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, purchaseMintAccount, user);
            const propertyData = this.propertyData;
            const propertyMintAccount = propertyData.propertyMintAccount;
            const propertyState = pda.property(propertyMintAccount).key;
            const userPropertyTokenAccount = yield spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, propertyMintAccount, user, true);
            const userPropertyTokenAccountInfo = yield this.ctx.connection.getAccountInfo(userPropertyTokenAccount, "confirmed");
            if (userPropertyTokenAccountInfo == null) {
                tx.addInstruction({
                    instructions: [
                        spl_token_1.Token.createAssociatedTokenAccountInstruction(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, propertyMintAccount, userPropertyTokenAccount, user, user),
                    ],
                    cleanupInstructions: [],
                    signers: [],
                });
            }
            const propertyVaultTokenAccount = yield spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, propertyMintAccount, propertyState, true);
            const platformVaultTokenAccount = yield spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, platformMintAccount, propertyState, true);
            const lockerAccount = pda.locker(user, propertyState).key;
            const timestamp = new anchor_1.BN(Math.floor(Date.now() / 1000));
            let purchasePropertyTokenIx;
            // private sale
            if (propertyData.privateSaleInfo.startTime.lte(timestamp) &&
                timestamp.lte(propertyData.privateSaleInfo.endTime)) {
                const userPlatformTokenAccount = yield spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, platformData.platformMintAccount, user, true);
                const userPlatformTokenAccountInfo = yield this.ctx.connection.getAccountInfo(userPlatformTokenAccount);
                if (userPlatformTokenAccountInfo == null) {
                    throw new Error(`6, Platform Token Account of ${user} not found`);
                }
                // without referrer
                if (!referrer) {
                    purchasePropertyTokenIx = yield this.ctx.methods.purchasePropertyToken({
                        accounts: {
                            user: this.ctx.provider.wallet.publicKey,
                            platformState: pda.platform().key,
                            platformMintAccount: platformData.platformMintAccount,
                            userPlatformTokenAccount,
                            platformVaultTokenAccount,
                            admin,
                            adminPurchaseTokenAccount,
                            purchaseMintAccount,
                            userPurchaseTokenAccount,
                            propertyState,
                            propertyMintAccount,
                            userPropertyTokenAccount,
                            propertyVaultTokenAccount,
                            lockerAccount,
                        },
                        inputs: {
                            purchaseAmount: new anchor_1.BN(purchaseAmount
                                .multipliedBy(new bignumber_js_1.default(10).pow(new bignumber_js_1.default(purchaseDecimals)))
                                .toString()),
                        },
                    });
                }
                // with referrer
                else {
                    const referrerPlatformTokenAccount = yield spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, platformData.platformMintAccount, referrer, true);
                    const referrerPlatformTokenAccountInfo = yield this.ctx.connection.getAccountInfo(referrerPlatformTokenAccount);
                    if (referrerPlatformTokenAccountInfo == null) {
                        tx.addInstruction({
                            instructions: [
                                spl_token_1.Token.createAssociatedTokenAccountInstruction(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, platformMintAccount, referrerPlatformTokenAccount, referrer, user),
                            ],
                            cleanupInstructions: [],
                            signers: [],
                        });
                    }
                    purchasePropertyTokenIx = yield this.ctx.methods.purchasePropertyToken({
                        accounts: {
                            user: this.ctx.provider.wallet.publicKey,
                            platformState: pda.platform().key,
                            platformMintAccount: platformData.platformMintAccount,
                            userPlatformTokenAccount,
                            platformVaultTokenAccount,
                            admin,
                            adminPurchaseTokenAccount,
                            purchaseMintAccount,
                            userPurchaseTokenAccount,
                            propertyState,
                            propertyMintAccount,
                            userPropertyTokenAccount,
                            propertyVaultTokenAccount,
                            lockerAccount,
                        },
                        remainingAccounts: [
                            {
                                isSigner: false,
                                isWritable: false,
                                pubkey: referrer,
                            },
                            {
                                isSigner: false,
                                isWritable: true,
                                pubkey: referrerPlatformTokenAccount,
                            },
                        ],
                        inputs: {
                            purchaseAmount: new anchor_1.BN(purchaseAmount
                                .multipliedBy(new bignumber_js_1.default(10).pow(new bignumber_js_1.default(purchaseDecimals)))
                                .toString()),
                        },
                    });
                }
            }
            // public sale
            else {
                const userPlatformTokenAccount = yield spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, platformData.platformMintAccount, user, true);
                const userPlatformTokenAccountInfo = yield this.ctx.connection.getAccountInfo(userPlatformTokenAccount);
                if (userPlatformTokenAccountInfo == null) {
                    tx.addInstruction({
                        instructions: [
                            spl_token_1.Token.createAssociatedTokenAccountInstruction(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, platformMintAccount, userPlatformTokenAccount, user, user),
                        ],
                        cleanupInstructions: [],
                        signers: [],
                    });
                }
                // without referrer
                if (!referrer) {
                    purchasePropertyTokenIx = yield this.ctx.methods.purchasePropertyToken({
                        accounts: {
                            user: this.ctx.provider.wallet.publicKey,
                            platformState: pda.platform().key,
                            platformMintAccount: platformData.platformMintAccount,
                            userPlatformTokenAccount,
                            platformVaultTokenAccount,
                            admin,
                            adminPurchaseTokenAccount,
                            purchaseMintAccount,
                            userPurchaseTokenAccount,
                            propertyState,
                            propertyMintAccount,
                            userPropertyTokenAccount,
                            propertyVaultTokenAccount,
                            lockerAccount,
                        },
                        inputs: {
                            purchaseAmount: new anchor_1.BN(purchaseAmount
                                .multipliedBy(new bignumber_js_1.default(10).pow(new bignumber_js_1.default(purchaseDecimals)))
                                .toString()),
                        },
                    });
                }
                // with referrer
                else {
                    const referrerPlatformTokenAccount = yield spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, platformData.platformMintAccount, referrer, true);
                    const referrerPlatformTokenAccountInfo = yield this.ctx.connection.getAccountInfo(referrerPlatformTokenAccount);
                    if (referrerPlatformTokenAccountInfo == null) {
                        tx.addInstruction({
                            instructions: [
                                spl_token_1.Token.createAssociatedTokenAccountInstruction(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, platformMintAccount, referrerPlatformTokenAccount, referrer, user),
                            ],
                            cleanupInstructions: [],
                            signers: [],
                        });
                    }
                    purchasePropertyTokenIx = yield this.ctx.methods.purchasePropertyToken({
                        accounts: {
                            user: this.ctx.provider.wallet.publicKey,
                            platformState: pda.platform().key,
                            platformMintAccount: platformData.platformMintAccount,
                            userPlatformTokenAccount,
                            platformVaultTokenAccount,
                            admin,
                            adminPurchaseTokenAccount,
                            purchaseMintAccount,
                            userPurchaseTokenAccount,
                            propertyState,
                            propertyMintAccount,
                            userPropertyTokenAccount,
                            propertyVaultTokenAccount,
                            lockerAccount,
                        },
                        remainingAccounts: [
                            {
                                isSigner: false,
                                isWritable: false,
                                pubkey: referrer,
                            },
                            {
                                isSigner: false,
                                isWritable: true,
                                pubkey: referrerPlatformTokenAccount,
                            },
                        ],
                        inputs: {
                            purchaseAmount: new anchor_1.BN(purchaseAmount
                                .multipliedBy(new bignumber_js_1.default(10).pow(new bignumber_js_1.default(purchaseDecimals)))
                                .toString()),
                        },
                    });
                }
            }
            if (purchasePropertyTokenIx.ix) {
                tx.addInstruction(purchasePropertyTokenIx.ix);
            }
            return tx;
        });
    }
    claimPropertyToken() {
        return __awaiter(this, void 0, void 0, function* () {
            const tx = new common_sdk_1.TransactionBuilder(this.ctx.provider.connection, this.ctx.provider.wallet);
            const user = this.ctx.provider.publicKey;
            const pda = this.pda;
            const propertyData = this.propertyData;
            const propertyMintAccount = propertyData.propertyMintAccount;
            const propertyState = pda.property(propertyMintAccount).key;
            const userPropertyTokenAccount = yield spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, propertyMintAccount, user, true);
            const userPropertyTokenAccountInfo = yield this.ctx.connection.getAccountInfo(userPropertyTokenAccount, "confirmed");
            if (userPropertyTokenAccountInfo == null) {
                tx.addInstruction({
                    instructions: [
                        spl_token_1.Token.createAssociatedTokenAccountInstruction(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, propertyMintAccount, userPropertyTokenAccount, user, user),
                    ],
                    cleanupInstructions: [],
                    signers: [],
                });
            }
            const propertyVaultTokenAccount = yield spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, propertyMintAccount, propertyState, true);
            const lockerAccount = pda.locker(user, propertyState).key;
            let claimPropertyTokenIx;
            claimPropertyTokenIx = yield this.ctx.methods.claimPropertyToken({
                accounts: {
                    user,
                    propertyState,
                    propertyMintAccount,
                    userPropertyTokenAccount,
                    propertyVaultTokenAccount,
                    lockerAccount,
                },
                inputs: {},
            });
            if (claimPropertyTokenIx.ix) {
                tx.addInstruction(claimPropertyTokenIx.ix);
            }
            return tx;
        });
    }
    getLocker() {
        return __awaiter(this, void 0, void 0, function* () {
            const user = this.ctx.provider.wallet.publicKey;
            const pda = this.pda;
            const propertyData = this.propertyData;
            const propertyMintAccount = propertyData.propertyMintAccount;
            const propertyMintAccountInfo = yield this.ctx.connection.getParsedAccountInfo(propertyMintAccount);
            if (propertyMintAccountInfo.value == null) {
                throw new Error(`7, Info of ${propertyMintAccount} not found`);
            }
            const propertyDecimals = propertyMintAccountInfo.value.data.parsed.info
                .decimals;
            const propertyState = pda.property(propertyMintAccount).key;
            const locker = pda.locker(user, propertyState).key;
            const lockerData = yield this.ctx.fetcher.getLocker(locker, true);
            if (!lockerData) {
                throw new Error(`8, Locker of ${user} not found`);
            }
            const uiPrivateSaleLocker = yield convertLockerDataToUiDataAndExtendData(lockerData, propertyDecimals, this.uiPropertyData);
            return uiPrivateSaleLocker;
        });
    }
    getCommissionAmount(purchaseAmount, hasReferral = false) {
        const now = Math.floor(Date.now() / 1000);
        const { privateSaleInfo, publicSaleInfo } = this.uiPropertyData;
        let commissionAmount = new bignumber_js_1.default(0);
        // private sale
        if (privateSaleInfo.startTime <= now && now <= privateSaleInfo.endTime) {
            const commissionFactor = hasReferral
                ? privateSaleInfo.referralCommissionAmount
                : privateSaleInfo.commissionAmount;
            commissionAmount = purchaseAmount.multipliedBy(commissionFactor);
        }
        // public sale
        else {
            const commissionFactor = purchaseAmount.gte(publicSaleInfo.referralCommissionBoostThreshold)
                ? hasReferral
                    ? publicSaleInfo.referralCommissionBoostAmount
                    : publicSaleInfo.commissionBoostAmount
                : hasReferral
                    ? publicSaleInfo.referralCommissionAmount
                    : publicSaleInfo.commissionAmount;
            commissionAmount = purchaseAmount.multipliedBy(commissionFactor);
        }
        return hasReferral ? commissionAmount.div(2) : commissionAmount;
    }
    convertTokenPurchasedEventDataToUiData(eventData) {
        return __awaiter(this, void 0, void 0, function* () {
            const pda = this.pda;
            const platformData = yield this.ctx.fetcher.getPlatform(pda.platform().key, true);
            if (!platformData) {
                throw new Error(`9, Platform state not found`);
            }
            const purchaseMintAccount = platformData.purchaseMintAccount;
            const platformMintAccount = platformData.platformMintAccount;
            const purchaseMintAccountInfo = yield this.ctx.connection.getParsedAccountInfo(purchaseMintAccount);
            if (purchaseMintAccountInfo.value == null) {
                throw new Error(`10, Info of ${purchaseMintAccount} not found`);
            }
            const purchaseDecimals = purchaseMintAccountInfo.value.data.parsed.info
                .decimals;
            const platformMintAccountInfo = yield this.ctx.connection.getParsedAccountInfo(platformMintAccount);
            if (platformMintAccountInfo.value == null) {
                throw new Error(`11, Info of ${platformMintAccount} not found`);
            }
            const platformDecimals = platformMintAccountInfo.value.data.parsed.info
                .decimals;
            const propertyData = this.propertyData;
            const propertyMintAccount = propertyData.propertyMintAccount;
            const propertyMintAccountInfo = yield this.ctx.connection.getParsedAccountInfo(propertyMintAccount);
            if (propertyMintAccountInfo.value == null) {
                throw new Error(`12, Info of ${propertyMintAccount} not found`);
            }
            const propertyDecimals = propertyMintAccountInfo.value.data.parsed.info
                .decimals;
            return Object.assign(Object.assign({}, eventData), { tokenPrice: new bignumber_js_1.default(eventData.tokenPrice.toString()).div(new bignumber_js_1.default(10).pow(purchaseDecimals)), purchasedAmount: new bignumber_js_1.default(eventData.purchasedAmount.toString()).div(new bignumber_js_1.default(10).pow(purchaseDecimals)), propertyAmount: new bignumber_js_1.default(eventData.propertyAmount.toString()).div(new bignumber_js_1.default(10).pow(propertyDecimals)), purchasedTimestamp: eventData.purchasedTimestamp.toNumber(), individualCommissionAmount: new bignumber_js_1.default(eventData.individualCommissionAmount.toString()).div(new bignumber_js_1.default(10).pow(platformDecimals)) });
        });
    }
    convertTokenClaimedEventDataToUiData(eventData) {
        return __awaiter(this, void 0, void 0, function* () {
            const propertyData = this.propertyData;
            const propertyMintAccount = propertyData.propertyMintAccount;
            const propertyMintAccountInfo = yield this.ctx.connection.getParsedAccountInfo(propertyMintAccount);
            if (propertyMintAccountInfo.value == null) {
                throw new Error(`13, Info of ${propertyMintAccount} not found`);
            }
            const propertyDecimals = propertyMintAccountInfo.value.data.parsed.info
                .decimals;
            return Object.assign(Object.assign({}, eventData), { claimedAmount: new bignumber_js_1.default(eventData.claimedAmount.toString()).div(new bignumber_js_1.default(10).pow(propertyDecimals)), claimedTimestamp: eventData.claimedTimestamp.toNumber() });
        });
    }
    refresh() {
        return __awaiter(this, void 0, void 0, function* () {
            const pda = this.pda;
            const property = pda.property(this.propertyMintAccount);
            const propertyData = yield this.ctx.fetcher.getProperty(property.key, true);
            if (!propertyData) {
                throw new Error(`14, Property of ${property.key} not found`);
            }
            this.propertyData = propertyData;
            this.uiPropertyData = yield convertPropertyDataToUiDataAndExtendData(this.ctx, this.pda, propertyData);
            return this;
        });
    }
}
exports.PropertyClient = PropertyClient;
function convertPropertyDataToUiDataAndExtendData(ctx, pda, propertyData) {
    return __awaiter(this, void 0, void 0, function* () {
        const { privateSaleInfo, publicSaleInfo, propertyMintAccount, bump } = propertyData;
        // get decimals of property token
        const propertyMintAccountInfo = yield ctx.connection.getParsedAccountInfo(propertyMintAccount);
        if (propertyMintAccountInfo.value == null) {
            throw new Error(`15, Info of ${propertyMintAccount} not found`);
        }
        const propertyDecimals = propertyMintAccountInfo.value.data.parsed.info
            .decimals;
        const propertyTotalSupply = new bignumber_js_1.default(propertyMintAccountInfo.value.data.parsed.info.supply);
        // get decimals of purchase token
        const platformData = yield ctx.fetcher.getPlatform(pda.platform().key, true);
        if (!platformData) {
            throw new Error(`16, Platform not found`);
        }
        const purchaseMintAccount = platformData.purchaseMintAccount;
        const purchaseMintAccountInfo = yield ctx.connection.getParsedAccountInfo(purchaseMintAccount);
        if (purchaseMintAccountInfo.value == null) {
            throw new Error(`17, Info of ${purchaseMintAccount} not found`);
        }
        const purchaseDecimals = purchaseMintAccountInfo.value.data.parsed.info
            .decimals;
        const platformMintAccount = platformData.platformMintAccount;
        const platformMintAccountInfo = yield ctx.connection.getParsedAccountInfo(platformMintAccount);
        if (platformMintAccountInfo.value == null) {
            throw new Error(`18, Info of ${platformMintAccount} not found`);
        }
        const platformDecimals = platformMintAccountInfo.value.data.parsed.info
            .decimals;
        const uiData = {
            privateSaleInfo: {
                startTime: Number(privateSaleInfo.startTime),
                endTime: Number(privateSaleInfo.endTime),
                supplyAmount: new bignumber_js_1.default(privateSaleInfo.supplyAmount.toString()).div(new bignumber_js_1.default(10).pow(new bignumber_js_1.default(propertyDecimals))),
                tokenPrice: new bignumber_js_1.default(privateSaleInfo.tokenPrice.toString()).div(new bignumber_js_1.default(10).pow(purchaseDecimals)),
                minimumPurchaseAmount: new bignumber_js_1.default(privateSaleInfo.minimumPurchaseAmount.toString()).div(new bignumber_js_1.default(10).pow(new bignumber_js_1.default(purchaseDecimals))),
                totalPurchasedAmount: new bignumber_js_1.default(privateSaleInfo.totalPurchasedAmount.toString()).div(new bignumber_js_1.default(10).pow(propertyDecimals)),
                commissionAmount: new bignumber_js_1.default(privateSaleInfo.commissionAmount.toString()).div(new bignumber_js_1.default(10).pow(platformDecimals)),
                referralCommissionAmount: new bignumber_js_1.default(privateSaleInfo.referralCommissionAmount.toString()).div(new bignumber_js_1.default(10).pow(platformDecimals)),
                discountPercentage: privateSaleInfo.discountPercentage / 100,
                tgePercentage: privateSaleInfo.tgePercentage / 100,
                unlockPeriod: Number(privateSaleInfo.unlockPeriod),
                unlockDuration: Number(privateSaleInfo.unlockDuration),
                minimumPlatformAmount: new bignumber_js_1.default(privateSaleInfo.minimumPlatformAmount.toString()).div(new bignumber_js_1.default(10).pow(platformDecimals)),
            },
            publicSaleInfo: {
                startTime: Number(publicSaleInfo.startTime),
                endTime: Number(publicSaleInfo.endTime),
                supplyAmount: new bignumber_js_1.default(publicSaleInfo.supplyAmount.toString()).div(new bignumber_js_1.default(10).pow(new bignumber_js_1.default(propertyDecimals))),
                tokenPrice: new bignumber_js_1.default(publicSaleInfo.tokenPrice.toString()).div(new bignumber_js_1.default(10).pow(purchaseDecimals)),
                minimumPurchaseAmount: new bignumber_js_1.default(publicSaleInfo.minimumPurchaseAmount.toString()).div(new bignumber_js_1.default(10).pow(new bignumber_js_1.default(purchaseDecimals))),
                totalPurchasedAmount: new bignumber_js_1.default(publicSaleInfo.totalPurchasedAmount.toString()).div(new bignumber_js_1.default(10).pow(propertyDecimals)),
                commissionAmount: new bignumber_js_1.default(publicSaleInfo.commissionAmount.toString()).div(new bignumber_js_1.default(10).pow(platformDecimals)),
                commissionBoostAmount: new bignumber_js_1.default(publicSaleInfo.commissionBoostAmount.toString()).div(new bignumber_js_1.default(10).pow(platformDecimals)),
                referralCommissionAmount: new bignumber_js_1.default(publicSaleInfo.referralCommissionAmount.toString()).div(new bignumber_js_1.default(10).pow(platformDecimals)),
                referralCommissionBoostAmount: new bignumber_js_1.default(publicSaleInfo.referralCommissionBoostAmount.toString()).div(new bignumber_js_1.default(10).pow(platformDecimals)),
                referralCommissionBoostThreshold: new bignumber_js_1.default(publicSaleInfo.referralCommissionBoostThreshold.toString()).div(new bignumber_js_1.default(10).pow(purchaseDecimals)),
            },
            propertyMintAccount: propertyMintAccount,
            totalSupply: propertyTotalSupply.div(new bignumber_js_1.default(10).pow(new bignumber_js_1.default(propertyDecimals))),
            bump,
        };
        return uiData;
    });
}
function convertLockerDataToUiDataAndExtendData(lockerData, propertyDecimals, uiPropertyData) {
    return __awaiter(this, void 0, void 0, function* () {
        const { privateSaleClaimedAmount, privateSaleTotalClaimableAmount, publicSaleTotalAmount } = lockerData;
        const uiPrivateSaleClaimedAmount = new bignumber_js_1.default(privateSaleClaimedAmount.toString()).div(new bignumber_js_1.default(10).pow(propertyDecimals));
        const uiPrivateSaleTotalClaimableAmount = new bignumber_js_1.default(privateSaleTotalClaimableAmount.toString()).div(new bignumber_js_1.default(10).pow(propertyDecimals));
        const { privateSaleInfo } = uiPropertyData;
        let privateSaleCurrentClaimableAmount = new bignumber_js_1.default(0);
        const timestamp = Math.floor(Date.now() / 1000);
        let nextUnlockTimestamp = privateSaleInfo.endTime + privateSaleInfo.unlockPeriod;
        if (timestamp >= privateSaleInfo.endTime + privateSaleInfo.unlockDuration) {
            if (uiPrivateSaleTotalClaimableAmount.gt(uiPrivateSaleClaimedAmount)) {
                privateSaleCurrentClaimableAmount = uiPrivateSaleTotalClaimableAmount.minus(uiPrivateSaleClaimedAmount);
            }
            nextUnlockTimestamp = 0;
        }
        else if (timestamp > privateSaleInfo.endTime) {
            const tgeAmount = uiPrivateSaleTotalClaimableAmount
                .multipliedBy(privateSaleInfo.tgePercentage)
                .dividedBy(100);
            const claimIndex = Math.floor((timestamp - privateSaleInfo.endTime) / privateSaleInfo.unlockPeriod);
            const privateSaleTotalClaimableAmountUntilCurrent = uiPrivateSaleTotalClaimableAmount
                .minus(tgeAmount)
                .multipliedBy((privateSaleInfo.unlockPeriod * claimIndex) / privateSaleInfo.unlockDuration)
                .plus(tgeAmount);
            if (privateSaleTotalClaimableAmountUntilCurrent.gt(uiPrivateSaleClaimedAmount)) {
                privateSaleCurrentClaimableAmount = privateSaleTotalClaimableAmountUntilCurrent.minus(uiPrivateSaleClaimedAmount);
            }
            nextUnlockTimestamp = privateSaleInfo.endTime + privateSaleInfo.unlockPeriod * (claimIndex + 1);
        }
        return {
            privateSaleClaimedAmount: uiPrivateSaleClaimedAmount,
            privateSaleTotalClaimableAmount: uiPrivateSaleTotalClaimableAmount,
            privateSaleCurrentClaimableAmount,
            privateSaleUnlockEndtime: privateSaleInfo.endTime + privateSaleInfo.unlockDuration,
            publicSaleTotalAmount: new bignumber_js_1.default(publicSaleTotalAmount.toString()).div(new bignumber_js_1.default(10).pow(propertyDecimals)),
            nextUnlockTimestamp,
        };
    });
}
